/*

Mquina Fotogrfica Robotizada

Ficheiro de base para usar para a implementao de Sistema de Regras de Produo
para o Controlo de uma Mquina Fotogrfica Robotizada

2010/11/02

*/


% :- ensure_loaded('C:/Users/luis/Cloud/Systems/psys.pl').

photo_facts_to_be_removed([objetivo/1, focado/1, fotografado/1, ligada/0, bateria_inserida/0,
			   bateria_carregada/0]).

:- photo_facts_to_be_removed(DPreds), dynamic(DPreds).

% Programa de controlo da mquibna robotizada

operar_maquina(Objectos, Estado):-
	write('A limpar a mermria.....'), nl,
	clearMemory(photo_facts_to_be_removed),
	write('Criar objetivos de fotografar objetos ....'), nl,
	processar_objectos_a_fotografar(Objectos),
	write('Inserir a definio do estado ...'), nl,
	processar_estado(Estado),
	write('Fim da inicializao...'), nl, nl,
	psys.


processar_objectos_a_fotografar([Obj|Objectos]):-
	criar_objetivo(fotografado(Obj)),
	processar_objectos_a_fotografar(Objectos).
processar_objectos_a_fotografar([]).


processar_estado([E|Estado]):-
	assert(E),processar_consequencias(E),
	processar_estado(Estado).
processar_estado([]).


processar_consequencias(ligada):-
	assert(bateria_carregada),
	assert(bateria_inserida),
	!.

processar_consequencias(focado(_)):-
	assert(ligada),
	processar_consequencias(ligada),
	!.
processar_consequencias(_).



% Definio de testes
test1 :-
	operar_maquina([1,2], [ligada]).

test2 :-
	operar_maquina([1,2], []).

test3 :-
	operar_maquina([1,2], [focado(2)]).

test4 :-
	operar_maquina([1,2], [bateria_inserida]).




% Acces ligadas  operao da mquina

fotografar :-
	focado(X),
	assert(fotografado(X)),
	writelist(['Fotografar ', X, '.']), nl.


focar(X):-
	retract(focado(_)),
	!,
	assert(focado(X)),
	writelist(['Focar ', X, '.']), nl.

focar(X):-
	assert(focado(X)),
	writelist(['Focar ', X, '.']), nl.


ligar:-
	assert(ligada),
	write('Ligar.'), nl.


desligar:-
	retract(ligada),
	write('Desligar.'), nl.


inserir_bateria:-
	assert(bateria_inserida),
	write('Inserir a bateria.'), nl.


carregar_bateria:-
	assert(bateria_carregada),
	write('Carregar a bateria.'), nl.


remover_bateria:-
	retract(bateria_inserida),
	write('Remover a bateria.'), nl.


criar_objetivo(P):-
	assert(objetivo(P)),
	writelist(['Criar o objectivo de atingir uma situao em que ', P, '  verdade.']), nl.


remover_objetivo(P):-
	retract(objetivo(P)),
	writelist(['Remover o objectivo de atingir uma situao em que ', P, '  verdade.']), nl.



% Aces relativas  especificao do estado inicial (interaco com o utilizador)
/*
perguntar_se(P, Resposta)

processar_resposta(P, Resposta)

armazenar(P)

assert(estado_especificado).

retract(estado_conhecido(E)).

*/

perguntar_se(P, Resposta):-
	writelist([P, ' eh verdade?']), nl,
	repeat,
		write('sim/nao: '),
		read(Resposta),
		member(Resposta, [sim, nao]), !.

processar_resposta(P, sim):-
	assert(P),
	assert(estado_conhecido(P)).
processar_resposta(P, nao):-
	assert(estado_conhecido(P)).

armazenar(P):-
	assert(P),
	assert(estado_conhecido(P)).



% Book keeping and stuff
clearMemory :-
	abolish(ligada/0),
	abolish(focado/1),
	abolish(fotografado/1),
	abolish(bateria_inserida/0),
	abolish(bateria_carregada/0),
	abolish(objectivo/1).


writelist([X|Rest]):-
	write(X),
	writelist(Rest).
writelist([]).











